;( function( $ ) {
	$( function() {
        var templates = {
            notice: '<div id="visma-message" class="updated notice notice-{{ type }} is-dismissible"><p>{{{ message }}}</p><button type="button" class="notice-dismiss"><span class="screen-reader-text">Dismiss this notice.</span></button></div>'
        };

        /**
         * AJAX Admin actions
         */
        $('.visma-admin-action').on('click', function (event) {
            event.preventDefault();

            var loader = $(this).siblings('.spinner');

            if (!$(this).data('visma-admin-action'))
                return console.warn("No bulk action specified.");

            loader.css({visibility: "visible"});
            var action = $(this).data('visma-admin-action');
            $.ajax({
                url: window.ajaxurl,
                data: {
                    action: "visma_admin_action",
                    visma_action: $(this).data('visma-admin-action')
                },
                success: function (response) {
                    loader.css({visibility: "hidden"});
                    if (action == 'visma_get_auth_url') {
                        window.open(response.message);
                    }

                    else if (action == 'visma_update_settings') {
                        if ("undefined" !== typeof response.message)
                            return alert(response.message);
                    }

                },
                dataType: "json",
                timeout: 3600000,
            });
        });

        /**
         * AJAX Admin actions
         */
        $('.visma-bulk-action').on('click', function (event) {
            event.preventDefault();

            var loader = $(this).siblings('.spinner');

            if (!$(this).data('visma-bulk-action'))
                return console.warn("No bulk action specified.");

            loader.css({visibility: "visible"});
            var action = $(this).data('visma-bulk-action');
            $.ajax({
                url: window.ajaxurl,
                data: {
                    action: "visma_bulk_action",
                    visma_action: $(this).data('visma-bulk-action')
                },
                success: function (response) {
                    loader.css({visibility: "hidden"});
                    if (action == 'visma_get_auth_url') {
                        window.open(response.message);
                    }

                    else if (action == 'visma_update_settings') {
                        if ("undefined" !== typeof response.message)
                            return alert(response.message);
                    }

                },
                dataType: "json",
                timeout: 3600000,
            });
        });

        /**
         * Sync order
         */
        $('.syncOrderToVisma').on('click', function (event) {
            event.preventDefault();

            var orderId = $(this).data('order-id');
            var nonce = $(this).data('nonce');
            var loader = $(this).siblings('.visma-spinner');
            var status = $(this).siblings('.wetail-visma-status');

            loader.css({visibility: "visible"});
            status.hide();

            $('#visma-message').remove();

            $.ajax({
                url: window.ajaxurl,
                data: {
                    action: "visma_action",
                    visma_action: "sync_order",
                    order_id: orderId
                },
                type: "POST",
                dataType: "json",
                success: function (response) {
                    if (!response.error)
                        status.removeClass('wetail-icon-cross').addClass('wetail-icon-check');

                    loader.css({visibility: "hidden"});
                    status.show();

                    $('#wpbody .wrap h1').after(Mustache.render(templates.notice, {
                        type: response.error ? "error" : "success",
                        message: response.message
                    }));

                    $('.sendInvoiceToCustomer[data-order-id=' + orderId + ']').removeClass("wetail-hidden");

                    $('html, body').animate({scrollTop: $('#visma-message').offset().top - 100});
                }
            });
        });

        /**
         * Sync product
         */
        $('.syncProductToVisma').on('click', function (event) {
            event.preventDefault();

            var productId = $(this).data('product-id');
            var nonce = $(this).data('nonce');
            var loader = $(this).siblings('.visma-spinner');
            var status = $(this).siblings('.wetail-visma-status');

            loader.css({visibility: "visible"});
            status.hide();

            $('#visma-message').remove();

            $.ajax({
                url: window.ajaxurl,
                data: {
                    action: "visma_action",
                    visma_action: "sync_product",
                    product_id: productId
                },
                type: "POST",
                dataType: "json",
                success: function (response) {
                    if (!response.error)
                        status.removeClass('wetail-icon-cross').addClass('wetail-icon-check');

                    loader.css({visibility: "hidden"});
                    status.show();

                    $('#wpbody .wrap h1').after(Mustache.render(templates.notice, {
                        type: response.error ? "error" : "success",
                        message: response.message
                    }));

                    $('html, body').animate({scrollTop: $('#visma-message').offset().top - 100});
                }
            });

            return;
        });

        function bulksyncProducts(productIds) {
            var nonce = $(this).data('nonce');
            var errors = 0;
            var index = 0;
            (function recursion_sync_all(productIds) {

                $.ajax({
                    url: ajaxurl,
                    data: {
                        'action': 'visma_action',
                        'visma_action': 'sync_product',
                        'nonce': nonce,
                        'product_id': productIds[index],
                    },
                    type: 'post',
                    success: function (data) {
                        index++;
                        $('#visma-message p').html("<p>Synchronizing product " + index + " of " + productIds.length + "</p>");
                        if (index >= productIds.length && errors === 0) {
                            all_products_synced_successfully(data);
                        } else {
                            recursion_sync_all(productIds);
                        }
                    },
                    error: function (a, b, error) {
                        index++;
                        $('#visma-message p').html("<p>Synchronizing product " + index + " of " + productIds.length + "</p>");
                        errors++;
                        if (index >= productIds.length && errors !== 0) {
                            there_was_errors(error);
                        } else {
                            recursion_sync_all(productIds);
                        }
                    }
                });

            })(productIds);

            function all_products_synced_successfully() {
                var loader = $(this).siblings('.visma-spinner');
                loader.css({visibility: "visible"});
            }

            function there_was_errors(error) {
                var loader = $(this).siblings('.visma-spinner');
                loader.css({visibility: "visible"});
            }

        }
    });
} )( jQuery );