<?php
namespace includes\api;

use Exception;
use includes\utils\WTV_Utils;
use includes\WTV_Plugin;
use includes\WTV_Settings_Validator;
use WC_Customer;
use WC_Order;
use WC_Shipping_Zones;
use includes\http\WTV_Request;

class WTV_Orders {

    /**
     * Sync order to Visma
     *
     * @param $order
     * @return mixed
     * @internal param int $order_id
     */
    public static function sync( $order ) {

        self::validate_order_items( $order );

        do_action('visma_before_order_sync', array( $order ));

        $customer_id = WTV_Customers::sync( $order );

        //try {
            $order_rows = [];
            $order_number = apply_filters( 'woocommerce_order_number', $order->get_id(), $order );
            $address = $order->get_address();
            $vat_number = self::has_eu_vat_number( $order );

            $visma_order = apply_filters( 'wetail_visma_order', [
                'CustomerId'                            => $customer_id,
                'Number'                                => preg_replace('/\D/', '', $order_number),
                'Amount'                                => $order->get_total(),
                'RoundingsAmount'                       => 0,
                'VatAmount'                             => $order->get_total_tax(),
                'Status'                                => self::format_status( $order->get_payment_method() ),
                'CustomerIsPrivatePerson'               => ( ! empty( $address['company'] ) ? false : true ),
                'OrderDate'                             => substr( $order->get_date_created(), 0, 10 ), # To cut off order time
                'VATIncluded'                           => apply_filters( 'wetail_visma_sync_order_vat_included', false ),
                'CurrencyCode'                          => $order->get_currency(),
                'DeliveryCustomerName'                  => ! empty( $address['company'] ) ? $address['company'] : $address['first_name'] . ' ' . $address['last_name'],
                'DeliveryAddress1'                      => $order->get_shipping_address_1(),
                'DeliveryAddress2'                      => $order->get_shipping_address_2(),
                'DeliveryPostalCode'                    => $order->get_shipping_postcode(),
                'DeliveryCity'                          => $order->get_shipping_city(),
                'DeliveryCountryCode'                   => $order->get_shipping_country() ? $order->get_shipping_country() : $order->get_billing_country(),
                'OurReference'                          => $order->get_shipping_first_name().' '.$order->get_shipping_last_name(),
                'InvoiceAddress1'                       => $order->get_billing_address_1(),
                'InvoiceAddress2'                       => $order->get_billing_address_2(),
                'InvoiceCity'                           => $order->get_billing_city(),
                'InvoiceCountryCode'                    => $order->get_billing_country(),
                'InvoiceCustomerName'                   => ! empty( $address['company'] ) ? $address['company'] : $address['first_name'] . ' ' . $address['last_name'],
                'InvoicePostalCode'                     => $order->get_billing_postcode(),
                'EuThirdParty'                          => $vat_number ? true : false ,
                'RotReducedInvoicingType'               => '0',
                'ReverseChargeOnConstructionServices'   => false,
            ], $order );

            if( 3 == $visma_order['Status']){
                $visma_order['ShippedDateTime'] = substr( $order->get_date_created(), 0, 10 );
            }

            $index = 1;
            foreach( $order->get_items() as $item ) {

                $order_row = self::create_order_row( $item, $order, $index );
                $index++;
                if ( isset( $order_row[0]) && is_array( $order_row[0] ) ) {
                    foreach ($order_row as $row) {
                        $order_rows[] = $row;
                    }
                }
                else{
                    $order_rows[] = $order_row;
                }
            }

            if( ! empty( $order->get_shipping_method() ) ){
                $shipping = self::get_shipping( $order, $index );

                if( is_array( $shipping ) ){
                    $order_rows[] = $shipping;
                }
            }


            $visma_order['Rows'] = $order_rows;

            # Create Order in Visma or update existing one
            $response = self::send_order_to_visma( $visma_order, $order->get_id() );
            # Set order as synced
            self::set_order_as_synced( $order, WTV_Plugin::SYNC_STATUS_ORDER_SYNCED );

        //} catch( Exception $error ) {
            //    throw new Exception( $error->getMessage() );
            //}

        // After order synced action
        do_action('visma_after_order_sync', $order , $visma_order );
        return true;
    }

    /**
     * Calculate item discount
     *
     * @param float $subtotal
     * @param float $total
     * @param int $quantity
     * @return mixed
     */
    public static function calculate_item_discount( $subtotal, $total, $quantity ){

        if ( $subtotal != $total ) {
            $item_discount = $subtotal - $total;
            return WTV_Utils::format_number( abs($item_discount/$subtotal ) );
        }
        return 0.0;
    }

    /**
     * Creates order row array
     *
     * @param \WC_Order_Item $item
     * @param WC_Order $order
     * @param $index
     * @return mixed
     * @throws Exception
     */
    public static function create_order_row( $item, $order, $index ){

        $product = ( self::item_is_variation( $item )) ? wc_get_product($item->get_variation_id()) : wc_get_product( $item->get_product_id() );
        $product_name = self::get_product_name( $item );

        if ( wc_get_product( $product->get_id() ) ) {
            if( get_option( 'visma_sync_existing_product') ){
                WTV_Products::sync( $product->get_id(), true );
            }
            else if ( ! WTV_Products::exists_in_visma( $product ) ){
                 WTV_Products::sync( $product->get_id(), true );
            }
            $product = wc_get_product( $product->get_id() );
        }

        // Cache quantity in var
        $quantity = $item->get_quantity();
        $subtotal = $order->get_item_subtotal( $item, false, false );

        // Get item total, not rounded
        $total = $order->get_item_total( $item, false, false );
        $order_row = apply_filters( 'wetail_visma_sync_modify_order_row', [
            'LineNumber'                    => $index,
            'ArticleId'                     => WTV_Products::get_visma_article_id( $product ),
            'ArticleNumber'                 => WTV_Products::sanitized_sku( $product->get_sku() ),
            'Text'                          => WTV_Products::sanitize_description( $product_name ),
            'DeliveredQuantity'             => floatval( $quantity ),
            'Quantity'                      => floatval( $quantity ),
            'IsTextRow'                     => false,
            'IsWorkCost'                    => false,
            'EligibleForReverseChargeOnVat' => false,
            'UnitPrice'                     => WTV_Utils::format_number( $subtotal ),
            'DiscountPercentage'            => self::calculate_item_discount( $subtotal, $total, $quantity ),
        ], $product, $item );

        // If the above filter returns an array of more rows,
        // we loop through them and add every row to $order_rows[].
        if (isset($order_row[0]) && is_array($order_row[0])) {
            $order_rows = array();
            foreach ($order_row as $row) {
                $order_rows[] = $row;
            }
            return $order_rows;
        }
        else {
            return $order_row;
        }
    }

    /**
     * Returns order from Visma
     *
     * @param int $order_id
     * @return mixed
     * @throws Exception
     */
    public static function get_order( $order_id ){
        $visma_order_id = wc_get_order( $order_id )->get_meta('visma_order_id');

        if( ! $visma_order_id ){
            throw new Exception( __( 'Order does not exist', WTV_Plugin::TEXTDOMAIN ) );
        }
        return WTV_Request::get( "/orders/{$visma_order_id}");
    }

    public static function get_product_name( $item ){
        if( self::item_is_variation( $item ) ){
            return preg_replace( array( '/Variation #\d* of /','/Variant #\d* av /'), '', $item->get_name()).' - '.wc_get_product( $item->get_variation_id() )->get_variation_attributes()[array_keys(wc_get_product($item->get_variation_id())->get_variation_attributes())[0]];
        }
        else{
            return $item->get_name();
        }
    }

    /**
     * @param WC_Order $order
     * @return bool
     */
    private static function has_eu_vat_number( $order ){

        if( $order->get_shipping_country() == 'SE' ){
            return false;
        }

        $vat_number = get_post_meta( $order->get_id(), '_vat_number', true );
        if ( ! empty( $vat_number ) ) {
            if ( wc_string_to_bool( get_post_meta( $order->get_id(), '_vat_number_is_valid', true) ) ) {
                return true;
            }
        }
        return false;
    }

    /**
     * Get Shipping
     *
     * @param \WC_Shipping $shipping_method
     * @param string $country_code
     * @return mixed
     * @throws Exception
     */
    public static function create_shipping_article( $shipping_method, $country_code, $shipping_method_name ){

	write_log('create_shipping_article');
        foreach( \WC_Tax::get_rates_for_tax_class( 'regular' ) as $tax_rate){
            if ( $country_code == $tax_rate->tax_rate_country ){
                $tax = $tax_rate->tax_rate;
            }
        }

        $article = [
            'IsActive'      => true,
            'Name'          => substr( $shipping_method_name, 0, 50),
            'Number'        => $shipping_method->get_method_id(),
            'CodingId'      => WTV_Visma_Settings::get_account_coding( $tax, $country_code ),
            'UnitId'        => WTV_Visma_Settings::get_standard_unit_id(),
            'NetPrice'      => 0,
        ];

        write_log('PAYLOAD' );
        write_log( $article );

        try{
            $response = WTV_Request::post( '/articles',  $article );
            $shipping_id = $response->Id;
        }
        catch ( Exception $error ){

            write_log('ERROR' );
            write_log( $error );

            if( $error->getCode() == WTV_Request::ERROR_CODE_DUPLICATE ){
                $filter = '$filter';
                $response = WTV_Request::get( "/articles?$filter=contains(Number,'" . $shipping_method->get_method_id() ."')",  $article );
                $shipping_id = $response->Data[0]->Id;
            }
        }

        write_log('RESPONSE' );
        write_log( $response );

        update_option( 'visma_shipping_' . $shipping_method->get_id(), $shipping_id );
        return $shipping_id;
    }

    /**
     * Get Shipping
     *
     * @param WC_Order $order
     * @param $index
     * @return mixed
     */
    public static function get_shipping( $order, $index ){

        $shipping_methods = $order->get_shipping_methods();
        $shipping_method = reset( $shipping_methods );
        $visma_shipping_article_id = get_option( 'visma_shipping_' . $shipping_method->get_id() );

        if( empty( $visma_shipping_article_id ) ){
            $visma_shipping_article_id = self::create_shipping_article( $shipping_method, $order->get_shipping_country(), $order->get_shipping_method() );
        }

        return apply_filters( 'wetail_visma_shipping', [
            'LineNumber'                    => $index,
            'ArticleId'                     => $visma_shipping_article_id,
            'Text'                          => $shipping_method->get_name(),
            'DeliveredQuantity'             => 1,
            'Quantity'                      => 1,
            'IsTextRow'                     => false,
            'IsWorkCost'                    => false,
            'EligibleForReverseChargeOnVat' => false,
            'UnitPrice'                     => $order->get_shipping_total() + $order->get_shipping_tax(),
        ], $order );
    }

    /**
     * Format status
     *
     * @param $payment_method
     * @return int
     * @internal param string $status
     */
    public static function format_status( $payment_method ){
        try{
            return get_option( 'visma_order_sync_settings_' . $payment_method )['order_status'];
        }
        catch ( Exception $e ){

        }

        return 1;
    }

    /**
     * Handle Fees
     *
     * @param WC_Order $order
     * @param mixed $visma_order
     * @return mixed
     */
    public static function handle_fees( $order, $visma_order ){

        # If order has a fee called 'Faktura'
        # add it as AdministrationFee in Visma
        foreach ($order->get_fees() as $fee) {
            if ($fee->get_name() == 'Faktura') {
                $invoice_fee = $order->get_item_total($fee, false, false);
            }
        }
        if (isset($invoice_fee))
            $visma_order['AdministrationFee'] = $invoice_fee;

        return $visma_order;
    }

    /**
     * Check wether order synced to Visma
     *
     * TODO: Change meta_key to '_visma_synced'
     *
     * @param int $order_id
     * @return mixed
     */
    public static function is_synced( $order_id ) {
        $synced = get_post_meta( $order_id, '_visma_order_synced', true );
        if( $synced ){
            return 1;
        }
    }

    /**
     * Checks if cart item is a variation
     *
     * @param mixed $item
     * @return boolean
     */
    public static function item_is_variation( $item ){
        if( ! empty( $item->get_variation_id() ) && wc_get_product($item->get_product_id()) ) {
            return true;
        }
        return false;
    }

    /**
     * Returns true if order exists in Visma.
     *
     * @param WC_Order $order
     * @return boolean
     */
    public static function order_exists( $order ){

        try {
            $response = self::get_order( $order->get_id() );
            return true;
        }
        catch( Exception $error ) {
            write_log( __( 'Order does not exist', WTV_Plugin::TEXTDOMAIN) );
            return false;
        }

    }

    /**
     * Set Way of Delivery.
     *
     * @param WC_Order $order
     * @param mixed $visma_order
     * @return mixed
     */
    public static function set_way_of_delivery( $order, $visma_order ){

        $shipping_zones = WC_Shipping_Zones::get_zones();
        $my_zone = 0;
        foreach( $shipping_zones as $zone ) {
            foreach ($zone['zone_locations'] as $zone_location) {
                if($order->get_shipping_country() == $zone_location->code){
                    $my_zone = $zone['zone_id'];
                    break;
                }
            }
        }
        $shipping = $order->get_items( 'shipping' );
        if( $shipping = $order->get_items( 'shipping' ) ) {
            $shipping = reset( $shipping );
            //preg_match('/(.*)(:\d*)/', $shipping['method_id'], $shipping_method_id);
            $visma_shipping_code = get_option( 'visma_shipping_code_' . $shipping['method_id'] .':'.$my_zone);
            if( $visma_shipping_code ) {
	            $visma_order['DeliveryMethodName'] = $visma_shipping_code;
            }
        }

        $delivery_terms = apply_filters( 'wetail_visma_way_and_terms_of_delivery', [
            'DeliveryMethodName'    => array_key_exists('DeliveryMethodName', $visma_order ) ? $visma_order['WayOfDelivery'] : '',
            'DeliveryMethodCode'    => array_key_exists('WayOfDelivery', $visma_order ) ? $visma_order['WayOfDelivery'] : '',
            'DeliveryTermName'      => array_key_exists('WayOfDelivery', $visma_order ) ? $visma_order['WayOfDelivery'] : '',
            'DeliveryTermCode'      => array_key_exists('WayOfDelivery', $visma_order ) ? $visma_order['WayOfDelivery'] : '',
        ], $shipping);
        $visma_order += $delivery_terms;
        return $visma_order;
    }

    /**
     * Sets postmeta '_visma_order_synced' of shop_order to true
     * @param WC_Order $order
     * @param int $sync_status
     */
    public static function set_order_as_synced( $order, $sync_status ) {
        $order->add_order_note( $sync_status == WTV_Plugin::SYNC_STATUS_ORDER_SYNCED ? __( 'Visma: Order synchronized', WTV_Plugin::TEXTDOMAIN ) : __( 'Visma: Voucher created', WTV_Plugin::TEXTDOMAIN ) );
        $order->save();
        update_post_meta( $order->get_id(), '_visma_order_synced', $sync_status );
    }

    /**
     * Sets postmeta 'visma_order_id' of shop_order
     * @param WC_Order $order
     * @param $visma_order_id
     */
    public static function set_visma_order_id( $order, $visma_order_id ) {
        $order->add_meta_data( 'visma_order_id', $visma_order_id );
        $order->save();
    }

    /**
     * Sends order to Visma either as POST or PUT
     * @param mixed $visma_order
     * @param mixed $order_number
     * @return mixed
     * @throws Exception
     */
    public static function send_order_to_visma( $visma_order, $order_id ){
        # Create Order in Visma or update existing one
        $order = wc_get_order( $order_id );

        if ( self::order_exists( $order ) ) {
            $visma_order_id = $order->get_meta( 'visma_order_id' );
            $response =WTV_Request::put( "/orders/{$visma_order_id}", $visma_order );
        }
        else {
            $response = WTV_Request::post( '/orders', $visma_order );
            self::set_visma_order_id( $order, $response->Id );
        }
        return $response;
    }

    /**
     * Checks every cart item for skus. Throws exception if not
     * @throws Exception
     * @param WC_Order $order
     * @return mixed
     */
    public static function validate_order_items( $order ){

	    if( get_option( 'visma_auto_generate_sku' ) ){
	    	return true;
	    }
        foreach( $order->get_items() as $item ) {
            $product_id = $item->get_product_id();
            $product = wc_get_product( $product_id );
            if ( ! $product->get_sku() ) {
                if( ! get_option( 'visma_sync_master_product' ) && $product->get_type() == 'variable' ){
                    continue;
                }

                throw new Exception( __( 'Product ID {$product_id} is missing SKU.', WTV_Plugin::TEXTDOMAIN ) );
            }
        }
        return true;
    }
}