<?php

namespace includes\api;

use includes\WTV_Plugin;
use includes\WTV_Settings_Validator;
use WC_Order;

/**
 * Created by PhpStorm.
 * User: tomas
 * Date: 2018-02-01
 * Time: 17:07
 */

class WTV_Sync_Controller{

    /**
     * Handles all syncing for an order
     * @param int $order_id
     * @return bool
     */
    public static function sync( $order_id ){

        $order = wc_get_order( $order_id );
        WTV_Settings_Validator::validate_settings_payment_method( $order->get_payment_method() );

        if( get_option('visma_sync_orders') ){
            WTV_Orders::sync( $order );
        }

        if( self::should_create_voucher( $order->get_payment_method() ) ){

            $voucher = new WTV_Vouchers( $order );
            $voucher->create_order_voucher();

            self::set_order_as_synced( $order, WTV_Plugin::SYNC_STATUS_VOUCHER_SYNCED);
        }
        return true;
    }

    public static function should_create_voucher( $payment_method ){
        $settings = get_option( 'visma_order_sync_settings_' . $payment_method );
        if ( $settings['voucher_series'] == 1000 ){
            return false;
        }
        return true;
    }

    /**
     * Sets postmeta '_visma_order_synced' of shop_order to true
     * @param $order
     * @param int $sync_status
     * @internal param int $order_id
     */
    public static function set_order_as_synced( $order, $sync_status ) {
        $order->add_order_note( $sync_status == WTV_Plugin::SYNC_STATUS_ORDER_SYNCED ? __( 'Visma: Order synchronized', WTV_Plugin::TEXTDOMAIN ) : __( 'Visma: Voucher created', WTV_Plugin::TEXTDOMAIN ) );
        $order->save();
        update_post_meta( $order->get_id(), '_visma_order_synced', $sync_status );
    }

}
