<?php
/**
 * Created by PhpStorm.
 * User: tomas
 * Date: 2018-01-24
 * Time: 09:44
 */

namespace includes;

use Exception;
use WC_Countries;

class WTV_Settings_Validator{

    /**
     * Check if all required settings are there
     */
    public static function all_settings_are_valid(){
        if ( get_option( 'visma_product_settings_are_valid' ) && get_option( 'visma_shipping_settings_are_valid' ) && get_option( 'visma_product_settings_are_valid' )  ){
            return true;
        }
        return false;
    }

    /**
     * Check all required order settings are there
     */
    public static function order_settings_are_valid(){
        if ( empty( get_option( 'visma_terms_of_payments', [] ) ) ){
            delete_option( 'visma_order_settings_are_valid' );
            return false;
        }
        update_option( 'visma_order_settings_are_valid', true );
        return true;
    }

    /**
     * Check if all required shipping settings are there
     */
    public static function shipping_settings_are_valid(){
        $setting = get_option( 'visma_shipping_account_se' );
        if ( ! $setting ) {
            delete_option( 'visma_shipping_settings_are_valid' );
            return false;
        }

        $country_settings = self::get_country_settings();

        if ( $country_settings['sells_to_eu'] ){
            $setting = get_option( 'visma_shipping_account_eu' );
            if ( ! $setting ) {
                delete_option( 'visma_shipping_settings_are_valid' );
                return false;
            }
        }

        if ( $country_settings['sells_to_world'] ){
            $setting = get_option( 'visma_shipping_account_world' );
            if ( ! $setting ) {
                delete_option( 'visma_shipping_settings_are_valid' );
                return false;
            }
        }

        update_option( 'visma_shipping_settings_are_valid', true );
        return true;

    }

    /**
     * Check if product settings are valid
     */
    public static function product_settings_are_valid(){

        $domestic_vat_array = [];

        $tax_settings = self::get_tax_settings();

        if( $tax_settings['25'] ){
            $domestic_vat_array[] = 'visma_domestic_vat_25_account';
        }

        if( $tax_settings['12'] ){
            $domestic_vat_array[] = 'visma_domestic_vat_12_account';
        }

        if( $tax_settings['6'] ){
            $domestic_vat_array[] = 'visma_domestic_vat_6_account';
        }

        foreach ( $domestic_vat_array as $domestic_vat ) {
            $setting = get_option( $domestic_vat );
            if ( ! $setting ) {
                delete_option( 'visma_product_settings_are_valid' );
                return false;
            }
        }

        $country_settings = self::get_country_settings();

        if ( $country_settings['sells_to_eu'] ){
            $eu_vat_array = [];
            if( $tax_settings['25'] ){
                $eu_vat_array[] = 'visma_eu_vat_25_account';
            }

            if( $tax_settings['12'] ){
                $eu_vat_array[] = 'visma_eu_vat_12_account';
            }

            if( $tax_settings['6'] ){
                $eu_vat_array[] = 'visma_eu_vat_6_account';
            }

            foreach ( $eu_vat_array as $eu_vat ) {
                $setting = get_option( $eu_vat );
                if ( ! $setting ) {
                    delete_option( 'visma_product_settings_are_valid' );
                    return false;
                }
            }
        }

        update_option( 'visma_product_settings_are_valid', true );
        return true;
    }


    /**
     * Returns which regions the shop is selling to
     * @return array
     */
    public static function get_country_settings( ){

        $settings = array(
            'sells_to_se'       => false,
            'sells_to_eu'       => false,
            'sells_to_world'    => false,
        );
        $countries = new WC_Countries();
        foreach( $countries->get_allowed_countries() as $country_code => $value){

            if ( $country_code == 'SE' ) {
                $settings['sells_to_se'] = true;
            }
            else if ( in_array( $country_code, $countries->get_european_union_countries() ) ){
                $settings['sells_to_eu'] = true;
            }
            else{
                $settings['sells_to_world'] = true;
            }

            if ( $settings['sells_to_eu'] && $settings['sells_to_world'] && $settings['sells_to_se']){
                return $settings;
            }
        }

        return $settings;
    }

    /**
     * Returns which tax rates the shop is selling to
     * @return array
     */
    public static function get_tax_settings( ){

        $settings = array(
            '25'    => false,
            '12'    => false,
            '6'     => false,
        );
        foreach( \WC_Tax::get_tax_classes() as $tax_class_name){
            foreach( \WC_Tax::get_rates_for_tax_class($tax_class_name) as $entry){
                if( 25 == intval( $entry->tax_rate ) ){
                    $settings['25'] = true;
                }
                elseif ( 12 == intval( $entry->tax_rate ) ){
                    $settings['12'] = true;
                }
                elseif ( 6 == intval( $entry->tax_rate ) ){
                    $settings['6'] = true;
                }
                if ( $settings['25'] && $settings['12'] && $settings['6'] ){
                    return $settings;
                }
            }
        }
        return $settings;
    }

    /**
     * Check if all required settings are there
     */
    public static function validate_settings(){
        if ( empty( get_option( 'visma_terms_of_payments', [] ) ) ){
            return false;
        }
        return true;
    }

    /**
     * Validates settings for payment method
     * @param string $payment_method
     * @throws Exception
     */
    public static function validate_settings_payment_method( $payment_method ){
        $settings = get_option( 'visma_order_sync_settings_' . $payment_method );

        if( empty( $settings ) ){
            throw new Exception( __( 'Inställningar saknas för ' . $payment_method, WTV_Plugin::TEXTDOMAIN ) );
        }

        if( !isset( $settings['payment_account'] ) || empty( $settings['payment_account'] ) ){
            throw new Exception( __( 'Bokföringskonto saknas för ' . $payment_method, WTV_Plugin::TEXTDOMAIN ) );
        }

        if( !isset( $settings['voucher_series'] )  || empty( $settings['voucher_series'] ) ){
            throw new Exception( __( 'Verifikationsserie saknas för ' . $payment_method, WTV_Plugin::TEXTDOMAIN ) );
        }
        if( get_option('visma_sync_orders') ){
            if( !isset( $settings['order_status'] ) || empty( $settings['order_status'] ) ){
                throw new Exception( __( 'Order status saknas för' . $payment_method, WTV_Plugin::TEXTDOMAIN ) );
            }

            if( !isset( $settings['terms_of_payment_id'] ) || empty( $settings['terms_of_payment_id'] ) ){
                throw new Exception( __( 'Betalningsvillkor saknas för' . $payment_method, WTV_Plugin::TEXTDOMAIN ) );
            }
        }
    }
}